@@ -13,7 +13,9 @@
 # limitations under the License.
 
 import collections
+import importlib.metadata
 import ipaddress
+import os
 import random
 import socket
 import threading
@@ -21,12 +23,9 @@ import time
 from urllib import parse as urlparse
 
 import eventlet
-from ironic_lib import exception as lib_exc
-from ironic_lib import mdns
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_log import log
-import pkg_resources
 
 from ironic_python_agent.api import app
 from ironic_python_agent import config
@@ -36,6 +35,8 @@ from ironic_python_agent.extensions import base
 from ironic_python_agent import hardware
 from ironic_python_agent import inspector
 from ironic_python_agent import ironic_api_client
+from ironic_python_agent import mdns
+from ironic_python_agent import netutils
 from ironic_python_agent import utils
 
 LOG = log.getLogger(__name__)
@@ -47,9 +48,6 @@ NETWORK_WAIT_TIMEOUT = 60
 # Time(in seconds) to wait before reattempt
 NETWORK_WAIT_RETRY = 5
 
-cfg.CONF.import_group('metrics', 'ironic_lib.metrics_utils')
-cfg.CONF.import_group('metrics_statsd', 'ironic_lib.metrics_statsd')
-
 Host = collections.namedtuple('Host', ['hostname', 'port'])
 
 
@@ -212,7 +210,7 @@ class IronicPythonAgent(base.ExecuteCommandMixin):
         if (not api_url or api_url == 'mdns') and not standalone:
             try:
                 api_url, params = mdns.get_endpoint('baremetal')
-            except lib_exc.ServiceLookupFailure:
+            except errors.ServiceLookupFailure:
                 if api_url:
                     # mDNS explicitly requested, report failure.
                     raise
@@ -233,8 +231,7 @@ class IronicPythonAgent(base.ExecuteCommandMixin):
         self.listen_address = listen_address
         self.advertise_address = advertise_address
         self.advertise_protocol = advertise_protocol
-        self.version = pkg_resources.get_distribution('ironic-python-agent')\
-            .version
+        self.version = importlib.metadata.version('ironic-python-agent')
         self.api = app.Application(self, cfg.CONF)
         self.heartbeat_timeout = None
         self.started_at = None
@@ -249,6 +246,9 @@ class IronicPythonAgent(base.ExecuteCommandMixin):
         self.hardware_initialization_delay = hardware_initialization_delay
         # IPA will stop serving requests and exit after this is set to False
         self.serve_api = True
+        # Together with serve_api, this option allows locking down the system
+        # before IPA stops.
+        self.lockdown = False
         self.agent_token = agent_token
         # Allows this to be turned on by the conductor while running,
         # in the event of long running ramdisks where the conductor
@@ -269,19 +269,11 @@ class IronicPythonAgent(base.ExecuteCommandMixin):
 
     def validate_agent_token(self, token):
         # We did not get a token, i.e. None and
-        # we've previously seen a token, which is
-        # a mid-cluster upgrade case with long-running ramdisks.
-        if (not token and self.agent_token
-                and not self.agent_token_required):
-            # TODO(TheJulia): Rip this out during or after the V cycle.
-            LOG.warning('Agent token for requests are not required '
-                        'by the conductor, yet we received a token. '
-                        'Cluster may be mid-upgrade. Support to '
-                        'not fail in this condition will be removed in '
-                        'the Victoria development cycle.')
-            # Tell the API everything is okay.
-            return True
-
+        # or we've not seen a token yet, so we
+        # cannot make a comparison, thus False.
+        if (not token or not self.agent_token):
+            return False
+        # Otherwise, compare the values.
         return self.agent_token == token
 
     def _get_route_source(self, dest):
@@ -313,15 +305,17 @@ class IronicPythonAgent(base.ExecuteCommandMixin):
         return source
 
     def _find_routable_addr(self):
-        ips = []
+        ips = set()
         for api_url in self.api_urls:
             ironic_host = urlparse.urlparse(api_url).hostname
             # Try resolving it in case it's not an IP address
             try:
-                ironic_host = socket.gethostbyname(ironic_host)
+                addrs = socket.getaddrinfo(ironic_host, 0)
             except socket.gaierror:
                 LOG.debug('Could not resolve %s, maybe no DNS', ironic_host)
-            ips.append(ironic_host)
+                ips.add(ironic_host)
+                continue
+            ips.update(addr for _, _, _, _, (addr, *_) in addrs)
 
         for attempt in range(self.ip_lookup_attempts):
             for ironic_host in ips:
@@ -563,7 +557,37 @@ class IronicPythonAgent(base.ExecuteCommandMixin):
                 LOG.error('Neither ipa-api-url nor inspection_callback_url'
                           'found, please check your pxe append parameters.')
 
-        self.serve_ipa_api()
+        in_rescued_mode = os.path.exists('/etc/.rescued')
+        if not in_rescued_mode:
+            self.serve_ipa_api()
+        else:
+            # NOTE(cid): In rescued state, we don't call _lockdown_system() as
+            # it brings down network interfaces which should be preserved for
+            # rescue operations.
+            LOG.info('Found rescue state marker file, locking down IPA '
+                     'in disabled mode')
+            self.heartbeater.stop()
+            self.serve_api = False
+            while True:
+                time.sleep(100)
 
         if not self.standalone and self.api_urls:
             self.heartbeater.stop()
+
+        if self.lockdown:
+            self._lockdown_system()
+            LOG.info('System locked down, looping forever to avoid a service '
+                     'restart')
+            while True:
+                time.sleep(100)
+
+    def _lockdown_system(self):
+        LOG.info('Locking down system after the API stopped')
+        # NOTE(dtantsur): not going through hardware managers here to minimize
+        # the amount of operations.
+        for iface in netutils.list_interfaces():
+            try:
+                utils.execute('ip', 'link', 'set', iface, 'down')
+            except Exception as exc:
+                LOG.warning('Could not bring down interface %s: %s',
+                            iface, exc)
