@@ -15,9 +15,12 @@
 from oslo_config import cfg
 from oslo_log import log as logging
 
+from ironic_python_agent.metrics_lib.metrics_statsd import statsd_opts
+from ironic_python_agent.metrics_lib.metrics_utils import metrics_opts
 from ironic_python_agent import netutils
 from ironic_python_agent import utils
 
+
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
@@ -253,7 +256,7 @@ cli_opts = [
     cfg.StrOpt('ntp_server',
                default=APARAMS.get('ipa-ntp-server', None),
                help='Address of a single NTP server against which the '
-                    'agent should sync the hardware clock prior to '
+                    'agent should sync the system software clock prior to '
                     'rebooting to an instance.'),
     cfg.BoolOpt('fail_if_clock_not_set',
                 default=False,
@@ -378,13 +381,18 @@ cli_opts = [
                      'security, but this option allows disabling it if there '
                      'is a compatibility concern.'),
     cfg.ListOpt('permitted_image_formats',
-                default='raw,qcow2',
+                default='raw,gpt,qcow2',
                 help='The supported list of image formats which are '
                      'permitted for deployment with Ironic Python Agent. If '
                      'an image format outside of this list is detected, the '
                      'image validation logic will fail the deployment '
                      'process. This check is skipped if deep image '
                      'inspection is disabled.'),
+    cfg.BoolOpt('disable_bootc_deploy',
+                default=False,
+                help='This disables bootc deployment methods in the ramdisk '
+                     'because the bootc command inside of the ramdisk '
+                     'comes from the supplied image to be deployed.'),
 ]
 
 disk_utils_opts = [
@@ -438,7 +446,9 @@ disk_part_opts = [
 def list_opts():
     return [('DEFAULT', cli_opts),
             ('disk_utils', disk_utils_opts),
-            ('disk_partitioner', disk_part_opts)]
+            ('disk_partitioner', disk_part_opts),
+            ('metrics', metrics_opts),
+            ('metrics_statsd', statsd_opts)]
 
 
 def populate_config():
@@ -446,6 +456,8 @@ def populate_config():
     CONF.register_cli_opts(cli_opts)
     CONF.register_opts(disk_utils_opts, group='disk_utils')
     CONF.register_opts(disk_part_opts, group='disk_partitioner')
+    CONF.register_opts(metrics_opts, group='metrics')
+    CONF.register_opts(statsd_opts, group='metrics_statsd')
 
 
 def override(params):
