@@ -15,7 +15,6 @@ import random
 import select
 import threading
 
-from ironic_lib import exception
 from oslo_config import cfg
 from oslo_log import log
 
@@ -90,7 +89,7 @@ class IronicInspection(threading.Thread):
                         interval = min(interval * self.backoff_factor,
                                        self.max_delay)
 
-                    except exception.ServiceLookupFailure as e:
+                    except errors.ServiceLookupFailure as e:
                         # Likely a mDNS lookup failure. We should
                         # keep retrying.
                         LOG.error('Error looking up introspection '
