@@ -12,15 +12,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import importlib.metadata
 import json
 import socket
 import time
 from unittest import mock
+from unittest.mock import sentinel
 
-from ironic_lib import exception as lib_exc
 from oslo_concurrency import processutils
 from oslo_config import cfg
-import pkg_resources
 from stevedore import extension
 
 from ironic_python_agent import agent
@@ -230,8 +230,8 @@ class TestBaseAgent(ironic_agent_base.IronicAgentTest):
         status = self.agent.get_status()
         self.assertIsInstance(status, agent.IronicPythonAgentStatus)
         self.assertEqual(started_at, status.started_at)
-        self.assertEqual(pkg_resources.get_distribution('ironic-python-agent')
-                         .version, status.version)
+        self.assertEqual(importlib.metadata.version('ironic-python-agent'),
+                         status.version)
 
     @mock.patch(
         'ironic_python_agent.hardware_managers.cna._detect_cna_card',
@@ -322,7 +322,7 @@ class TestBaseAgent(ironic_agent_base.IronicAgentTest):
         self.agent.heartbeater.start.assert_called_once_with()
         self.assertTrue(CONF.md5_enabled)
 
-    @mock.patch('ironic_lib.mdns.get_endpoint', autospec=True)
+    @mock.patch('ironic_python_agent.mdns.get_endpoint', autospec=True)
     @mock.patch(
         'ironic_python_agent.hardware_managers.cna._detect_cna_card',
         mock.Mock())
@@ -377,7 +377,7 @@ class TestBaseAgent(ironic_agent_base.IronicAgentTest):
                          mock_dispatch.call_args_list)
         self.agent.heartbeater.start.assert_called_once_with()
 
-    @mock.patch('ironic_lib.mdns.get_endpoint', autospec=True)
+    @mock.patch('ironic_python_agent.mdns.get_endpoint', autospec=True)
     @mock.patch(
         'ironic_python_agent.hardware_managers.cna._detect_cna_card',
         mock.Mock())
@@ -624,7 +624,7 @@ class TestBaseAgent(ironic_agent_base.IronicAgentTest):
         self.agent.heartbeater.start.assert_called_once_with()
 
     @mock.patch.object(hardware, '_enable_multipath', autospec=True)
-    @mock.patch('ironic_lib.mdns.get_endpoint', autospec=True)
+    @mock.patch('ironic_python_agent.mdns.get_endpoint', autospec=True)
     @mock.patch(
         'ironic_python_agent.hardware_managers.cna._detect_cna_card',
         mock.Mock())
@@ -643,7 +643,7 @@ class TestBaseAgent(ironic_agent_base.IronicAgentTest):
                                                 mock_wait,
                                                 mock_mdns,
                                                 mock_mpath):
-        mock_mdns.side_effect = lib_exc.ServiceLookupFailure()
+        mock_mdns.side_effect = errors.ServiceLookupFailure()
         # If inspection_callback_url is configured and api_url is not when the
         # agent starts, ensure that the inspection will be called and wsgi
         # server will work as usual. Also, make sure api_client and heartbeater
@@ -683,7 +683,7 @@ class TestBaseAgent(ironic_agent_base.IronicAgentTest):
         self.assertFalse(mock_dispatch.called)
 
     @mock.patch.object(hardware, '_enable_multipath', autospec=True)
-    @mock.patch('ironic_lib.mdns.get_endpoint', autospec=True)
+    @mock.patch('ironic_python_agent.mdns.get_endpoint', autospec=True)
     @mock.patch(
         'ironic_python_agent.hardware_managers.cna._detect_cna_card',
         mock.Mock())
@@ -702,7 +702,7 @@ class TestBaseAgent(ironic_agent_base.IronicAgentTest):
                                                mock_wait,
                                                mock_mdns,
                                                mock_mpath):
-        mock_mdns.side_effect = lib_exc.ServiceLookupFailure()
+        mock_mdns.side_effect = errors.ServiceLookupFailure()
         # If both api_url and inspection_callback_url are not configured when
         # the agent starts, ensure that the inspection will be skipped and wsgi
         # server will work as usual. Also, make sure api_client and heartbeater
@@ -740,6 +740,65 @@ class TestBaseAgent(ironic_agent_base.IronicAgentTest):
         self.assertTrue(mock_wait.called)
         self.assertFalse(mock_dispatch.called)
 
+    @mock.patch.object(time, 'sleep', autospec=True)
+    @mock.patch.object(utils, 'execute', autospec=True)
+    @mock.patch.object(netutils, 'list_interfaces', autospec=True)
+    @mock.patch(
+        'ironic_python_agent.hardware_managers.cna._detect_cna_card',
+        mock.Mock())
+    @mock.patch.object(hardware, 'dispatch_to_managers', autospec=True)
+    @mock.patch.object(agent.IronicPythonAgent,
+                       '_wait_for_interface', autospec=True)
+    @mock.patch('oslo_service.wsgi.Server', autospec=True)
+    @mock.patch.object(hardware, 'get_managers', autospec=True)
+    def test_run_then_lockdown(self, mock_get_managers, mock_wsgi,
+                               mock_wait, mock_dispatch, mock_interfaces,
+                               mock_exec, mock_sleep):
+        CONF.set_override('inspection_callback_url', '')
+
+        wsgi_server = mock_wsgi.return_value
+
+        def set_serve_api():
+            self.agent.lockdown = True
+            self.agent.serve_api = False
+
+        wsgi_server.start.side_effect = set_serve_api
+        self.agent.heartbeater = mock.Mock()
+        self.agent.api_client.lookup_node = mock.Mock()
+        self.agent.api_client.lookup_node.return_value = {
+            'node': {
+                'uuid': 'deadbeef-dabb-ad00-b105-f00d00bab10c'
+            },
+            'config': {
+                'heartbeat_timeout': 300,
+                'agent_md5_checksum_enable': False
+            }
+        }
+        mock_interfaces.return_value = ['em1', 'em2']
+
+        class StopTesting(Exception):
+            """Exception to exit the infinite loop."""
+
+        mock_sleep.side_effect = StopTesting
+
+        self.assertRaises(StopTesting, self.agent.run)
+
+        mock_wsgi.assert_called_once_with(CONF, 'ironic-python-agent',
+                                          app=self.agent.api,
+                                          host=mock.ANY, port=9999,
+                                          use_ssl=False)
+        wsgi_server.start.assert_called_once_with()
+        mock_wait.assert_called_once_with(mock.ANY)
+        self.assertEqual([mock.call('list_hardware_info'),
+                          mock.call('wait_for_disks')],
+                         mock_dispatch.call_args_list)
+        self.agent.heartbeater.start.assert_called_once_with()
+        self.agent.heartbeater.stop.assert_called_once_with()
+        mock_exec.assert_has_calls([
+            mock.call('ip', 'link', 'set', iface, 'down')
+            for iface in ['em1', 'em2']
+        ])
+
     @mock.patch.object(time, 'time', autospec=True)
     @mock.patch.object(time, 'sleep', autospec=True)
     @mock.patch.object(hardware, 'dispatch_to_managers', autospec=True)
@@ -999,7 +1058,7 @@ class TestAgentStandalone(ironic_agent_base.IronicAgentTest):
 @mock.patch.object(hardware, '_load_ipmi_modules', lambda: None)
 @mock.patch.object(hardware.GenericHardwareManager, 'wait_for_disks',
                    lambda self: None)
-@mock.patch.object(socket, 'gethostbyname', autospec=True)
+@mock.patch.object(socket, 'getaddrinfo', autospec=True)
 @mock.patch.object(utils, 'execute', autospec=True)
 class TestAdvertiseAddress(ironic_agent_base.IronicAgentTest):
     def setUp(self):
@@ -1017,14 +1076,14 @@ class TestAdvertiseAddress(ironic_agent_base.IronicAgentTest):
             agent_token=None,
             standalone=False)
 
-    def test_advertise_address_provided(self, mock_exec, mock_gethostbyname):
+    def test_advertise_address_provided(self, mock_exec, mock_getaddrinfo):
         self.agent.advertise_address = agent.Host('1.2.3.4', 9990)
 
         self.agent.set_agent_advertise_addr()
 
         self.assertEqual(('1.2.3.4', 9990), self.agent.advertise_address)
         self.assertFalse(mock_exec.called)
-        self.assertFalse(mock_gethostbyname.called)
+        self.assertFalse(mock_getaddrinfo.called)
 
     @mock.patch.object(hardware, '_enable_multipath', autospec=True)
     @mock.patch.object(netutils, 'get_ipv4_addr',
@@ -1032,7 +1091,7 @@ class TestAdvertiseAddress(ironic_agent_base.IronicAgentTest):
     @mock.patch('ironic_python_agent.hardware_managers.cna._detect_cna_card',
                 autospec=True)
     def test_with_network_interface(self, mock_cna, mock_get_ipv4, mock_mpath,
-                                    mock_exec, mock_gethostbyname):
+                                    mock_exec, mock_getaddrinfo):
         self.agent.network_interface = 'em1'
         mock_get_ipv4.return_value = '1.2.3.4'
         mock_cna.return_value = False
@@ -1043,7 +1102,7 @@ class TestAdvertiseAddress(ironic_agent_base.IronicAgentTest):
                          self.agent.advertise_address)
         mock_get_ipv4.assert_called_once_with('em1')
         self.assertFalse(mock_exec.called)
-        self.assertFalse(mock_gethostbyname.called)
+        self.assertFalse(mock_getaddrinfo.called)
 
     @mock.patch.object(hardware, '_enable_multipath', autospec=True)
     @mock.patch.object(netutils, 'get_ipv4_addr',
@@ -1052,7 +1111,7 @@ class TestAdvertiseAddress(ironic_agent_base.IronicAgentTest):
                 autospec=True)
     def test_with_network_interface_failed(self, mock_cna, mock_get_ipv4,
                                            mock_mpath, mock_exec,
-                                           mock_gethostbyname):
+                                           mock_getaddrinfo):
         self.agent.network_interface = 'em1'
         mock_get_ipv4.return_value = None
         mock_cna.return_value = False
@@ -1062,11 +1121,11 @@ class TestAdvertiseAddress(ironic_agent_base.IronicAgentTest):
 
         mock_get_ipv4.assert_called_once_with('em1')
         self.assertFalse(mock_exec.called)
-        self.assertFalse(mock_gethostbyname.called)
+        self.assertFalse(mock_getaddrinfo.called)
 
-    def test_route_with_ip(self, mock_exec, mock_gethostbyname):
+    def test_route_with_ip(self, mock_exec, mock_getaddrinfo):
         self.agent.api_urls = ['http://1.2.1.2:8081/v1']
-        mock_gethostbyname.side_effect = socket.gaierror()
+        mock_getaddrinfo.side_effect = socket.gaierror()
         mock_exec.return_value = (
             """1.2.1.2 via 192.168.122.1 dev eth0  src 192.168.122.56
                 cache """,
@@ -1078,11 +1137,11 @@ class TestAdvertiseAddress(ironic_agent_base.IronicAgentTest):
         self.assertEqual(('192.168.122.56', 9990),
                          self.agent.advertise_address)
         mock_exec.assert_called_once_with('ip', 'route', 'get', '1.2.1.2')
-        mock_gethostbyname.assert_called_once_with('1.2.1.2')
+        mock_getaddrinfo.assert_called_once_with('1.2.1.2', 0)
 
-    def test_route_with_ipv6(self, mock_exec, mock_gethostbyname):
+    def test_route_with_ipv6(self, mock_exec, mock_getaddrinfo):
         self.agent.api_urls = ['http://[fc00:1111::1]:8081/v1']
-        mock_gethostbyname.side_effect = socket.gaierror()
+        mock_getaddrinfo.side_effect = socket.gaierror()
         mock_exec.return_value = (
             """fc00:101::1 dev br-ctlplane  src fc00:101::4  metric 0
                 cache """,
@@ -1094,10 +1153,13 @@ class TestAdvertiseAddress(ironic_agent_base.IronicAgentTest):
         self.assertEqual(('fc00:101::4', 9990),
                          self.agent.advertise_address)
         mock_exec.assert_called_once_with('ip', 'route', 'get', 'fc00:1111::1')
-        mock_gethostbyname.assert_called_once_with('fc00:1111::1')
+        mock_getaddrinfo.assert_called_once_with('fc00:1111::1', 0)
 
-    def test_route_with_host(self, mock_exec, mock_gethostbyname):
-        mock_gethostbyname.return_value = '1.2.1.2'
+    def test_route_with_host(self, mock_exec, mock_getaddrinfo):
+        mock_getaddrinfo.return_value = [
+            (sentinel.a, sentinel.b, sentinel.c, sentinel.d,
+             ('1.2.1.2', sentinel.e)),
+        ]
         mock_exec.return_value = (
             """1.2.1.2 via 192.168.122.1 dev eth0  src 192.168.122.56
                 cache """,
@@ -1109,11 +1171,32 @@ class TestAdvertiseAddress(ironic_agent_base.IronicAgentTest):
         self.assertEqual(('192.168.122.56', 9990),
                          self.agent.advertise_address)
         mock_exec.assert_called_once_with('ip', 'route', 'get', '1.2.1.2')
-        mock_gethostbyname.assert_called_once_with('fake_api.example.org')
+        mock_getaddrinfo.assert_called_once_with('fake_api.example.org', 0)
+
+    def test_route_with_host_v6(self, mock_exec, mock_getaddrinfo):
+        mock_getaddrinfo.return_value = [
+            (sentinel.a, sentinel.b, sentinel.c, sentinel.d,
+             ('fc00:1111::1', sentinel.e)),
+        ]
+        mock_exec.return_value = (
+            """fc00:101::1 dev br-ctlplane  src fc00:101::4  metric 0
+                cache """,
+            ""
+        )
+
+        self.agent.set_agent_advertise_addr()
+
+        self.assertEqual(('fc00:101::4', 9990),
+                         self.agent.advertise_address)
+        mock_exec.assert_called_once_with('ip', 'route', 'get', 'fc00:1111::1')
+        mock_getaddrinfo.assert_called_once_with('fake_api.example.org', 0)
 
     @mock.patch.object(time, 'sleep', autospec=True)
-    def test_route_retry(self, mock_sleep, mock_exec, mock_gethostbyname):
-        mock_gethostbyname.return_value = '1.2.1.2'
+    def test_route_retry(self, mock_sleep, mock_exec, mock_getaddrinfo):
+        mock_getaddrinfo.return_value = [
+            (sentinel.a, sentinel.b, sentinel.c, sentinel.d,
+             ('1.2.1.2', sentinel.e)),
+        ]
         mock_exec.side_effect = [
             processutils.ProcessExecutionError('boom'),
             (
@@ -1132,15 +1215,18 @@ class TestAdvertiseAddress(ironic_agent_base.IronicAgentTest):
         self.assertEqual(('192.168.122.56', 9990),
                          self.agent.advertise_address)
         mock_exec.assert_called_with('ip', 'route', 'get', '1.2.1.2')
-        mock_gethostbyname.assert_called_once_with('fake_api.example.org')
+        mock_getaddrinfo.assert_called_once_with('fake_api.example.org', 0)
         mock_sleep.assert_called_with(10)
         self.assertEqual(3, mock_exec.call_count)
         self.assertEqual(2, mock_sleep.call_count)
 
     @mock.patch.object(time, 'sleep', autospec=True)
     def test_route_several_urls_and_retries(self, mock_sleep, mock_exec,
-                                            mock_gethostbyname):
-        mock_gethostbyname.side_effect = lambda x: x
+                                            mock_getaddrinfo):
+        mock_getaddrinfo.side_effect = lambda addr, port: [
+            (sentinel.a, sentinel.b, sentinel.c, sentinel.d,
+             (addr, sentinel.e)),
+        ]
         self.agent.api_urls = ['http://[fc00:1111::1]:8081/v1',
                                'http://1.2.1.2:8081/v1']
         mock_exec.side_effect = [
@@ -1161,32 +1247,38 @@ class TestAdvertiseAddress(ironic_agent_base.IronicAgentTest):
 
         self.assertEqual(('192.168.122.56', 9990),
                          self.agent.advertise_address)
-        mock_exec.assert_has_calls([
-            mock.call('ip', 'route', 'get', 'fc00:1111::1'),
-            mock.call('ip', 'route', 'get', '1.2.1.2'),
-            mock.call('ip', 'route', 'get', 'fc00:1111::1'),
-            mock.call('ip', 'route', 'get', '1.2.1.2'),
-        ])
-        mock_gethostbyname.assert_has_calls([
-            mock.call('fc00:1111::1'),
-            mock.call('1.2.1.2'),
+        self.assertCountEqual(
+            mock_exec.mock_calls,
+            [
+                mock.call('ip', 'route', 'get', 'fc00:1111::1'),
+                mock.call('ip', 'route', 'get', '1.2.1.2'),
+                mock.call('ip', 'route', 'get', 'fc00:1111::1'),
+                mock.call('ip', 'route', 'get', '1.2.1.2'),
+            ],
+        )
+        mock_getaddrinfo.assert_has_calls([
+            mock.call('fc00:1111::1', 0),
+            mock.call('1.2.1.2', 0),
         ])
         mock_sleep.assert_called_with(10)
         self.assertEqual(4, mock_exec.call_count)
         # Both URLs are handled in a single attempt, so only one sleep here
         self.assertEqual(1, mock_sleep.call_count)
-        self.assertEqual(2, mock_gethostbyname.call_count)
+        self.assertEqual(2, mock_getaddrinfo.call_count)
 
     @mock.patch.object(time, 'sleep', autospec=True)
-    def test_route_failed(self, mock_sleep, mock_exec, mock_gethostbyname):
-        mock_gethostbyname.return_value = '1.2.1.2'
+    def test_route_failed(self, mock_sleep, mock_exec, mock_getaddrinfo):
+        mock_getaddrinfo.return_value = [
+            (sentinel.a, sentinel.b, sentinel.c, sentinel.d,
+             ('1.2.1.2', sentinel.e)),
+        ]
         mock_exec.side_effect = processutils.ProcessExecutionError('boom')
 
         self.assertRaises(errors.LookupAgentIPError,
                           self.agent.set_agent_advertise_addr)
 
         mock_exec.assert_called_with('ip', 'route', 'get', '1.2.1.2')
-        mock_gethostbyname.assert_called_once_with('fake_api.example.org')
+        mock_getaddrinfo.assert_called_once_with('fake_api.example.org', 0)
         self.assertEqual(5, mock_exec.call_count)
         self.assertEqual(5, mock_sleep.call_count)
 
